package service

import (
	"context"
	"github.com/spf13/cast"
	"gorm.io/gorm"
	"hilo/api/service/mgr"
	"hilo/app/service/user/internal/model"
	"hilo/common/mysql"
)

func (s *UserService) GetUserDetail(ctx context.Context, user *model.User) (*model.CvUserDetail, error) {
	userId := user.Id
	//统计喜欢
	var likeN int64
	err := s.dao.DB.Model(&model.UserLike{}).Where(&model.UserLike{UserId: userId}).Count(&likeN).Error
	if err != nil {
		return nil, err
	}

	//统计是否存在会话
	var sessionN int64
	err = s.dao.DB.Model(&model.SessionView{}).Where(&model.SessionView{OtherUserId: userId}).Count(&sessionN).Error
	if err != nil {
		return nil, err
	}

	var likeMe int64
	err = s.dao.DB.Model(&model.UserLike{}).Where(&model.UserLike{LikeUserId: userId}).Count(&likeMe).Error
	if err != nil {
		return nil, err
	}

	var wealthUserScore *model.MatchWealthUserScore
	if err = s.dao.DB.Model(&model.MatchWealthUserScore{}).Where(&model.MatchWealthUserScore{UserId: userId}).First(&wealthUserScore).Error; err != nil {
		if err != nil {
			if err == gorm.ErrRecordNotFound {
				wealthUserScore = &model.MatchWealthUserScore{
					UserId: userId,
					Score:  0,
					Grade:  0,
				}
			} else {
				return nil, err
			}
		}
	}

	var charmUserScore model.MatchCharmUserScore
	if err = s.dao.DB.Model(&model.MatchCharmUserScore{}).Where(model.MatchCharmUserScore{
		UserId: userId,
	}).First(&charmUserScore).Error; err != nil {
		if err != nil {
			if err == gorm.ErrRecordNotFound {
				charmUserScore = model.MatchCharmUserScore{
					UserId: userId,
					Score:  0,
					Grade:  0,
				}
			} else {
				return nil, err
			}
		}
	}

	var activityUserScore model.MatchActityUserScore
	if err = s.dao.DB.Model(&model.MatchActityUserScore{}).Where(&model.MatchActityUserScore{
		UserId: userId,
	}).First(&activityUserScore).Error; err != nil {
		if err != nil {
			if err == gorm.ErrRecordNotFound {
				activityUserScore = model.MatchActityUserScore{
					UserId: userId,
					Score:  0,
					Grade:  0,
				}
			} else {
				return nil, err
			}
		}
	}

	isVip, expireTime, err := s.IsVip(userId)
	if err != nil {
		return nil, err
	}
	headwear, err := s.GetCvHeadwear(userId)
	if err != nil {
		return nil, err
	}

	medals := make(map[uint64][]int, 0)
	medal, err := s.dao.GetUserMedal(userId)
	if err != nil {
		return nil, err
	}
	medals[uint64(userId)] = medal

	medals, medalInfo, err := s.getMedalInfoMap(medals)
	if err != nil {
		return nil, err
	}

	//rooms TODO

	groupPowerId, groupPowerName, err := s.GetUserGroupPowerInfo(userId)
	if err != nil {
		return nil, err
	}

	//mp, err := s.dao.BatchGetPropertyMapByUserIds([]int64{userId})
	//if err != nil {
	//	return nil, err
	//}

	// properties TODO

	ride := &model.CvProperty{}

	noble, err := s.FindActiveNoble(userId)
	if err != nil {
		return nil, err
	}

	userTradeUnion, err := s.GetUserTradeUnion(userId)
	if err != nil {
		return nil, err
	}

	superManagerMap, err := s.GetSuperManagerMap([]int64{userId})
	if err != nil {
		return nil, err
	}

	cvUserDetail := &model.CvUserDetail{
		CvUserBase: model.CvUserBase{
			Id:              user.Id,
			Avatar:          IfLogoutStr(IfLogout(user.LogoutTime), "", user.Avatar),
			DefaultAvatar:   cast.ToBool(user.DefaultAvatar),
			ExternalId:      user.ExternalId,
			Nick:            IfLogoutStr(IfLogout(user.LogoutTime), user.Code, user.Nick),
			Description:     IfLogoutStr(IfLogout(user.LogoutTime), "", user.Description),
			Sex:             uint8(user.Sex),
			Country:         user.Country,
			CountryIcon:     user.CountryIcon,
			Code:            user.Code,
			IsPrettyCode:    user.IsPrettyCode(),
			IsVip:           isVip,
			IsOfficialStaff: superManagerMap[userId],
			VipExpireTime:   expireTime,
			Medals:          IfLogoutMedals(IfLogout(user.LogoutTime), []int{}, medals[uint64(userId)]),
			MedalInfo:       IfLogoutMedalInfo(IfLogout(user.LogoutTime), []*model.CvMedal{}, medalInfo[uint64(userId)]),
			Headwear:        IfLogoutHeadwear(IfLogout(user.LogoutTime), nil, headwear),
			Ride:            IfLogoutRide(IfLogout(user.LogoutTime), &model.CvProperty{}, ride),
		},
		IsPush:            uint8(user.IsPush),
		IsLike:            likeN > 0,
		IsLikeMe:          likeMe > 0,
		WealthUserGrade:   wealthUserScore.Grade,
		CharmUserGrade:    charmUserScore.Grade,
		ActivityUserGrade: activityUserScore.Grade,
		CurrentRoom:       "0",
		MyGroupPower:      groupPowerId,
		MyGroupPowerName:  groupPowerName,
	}
	if noble != nil {
		cvUserDetail.Noble = &model.CvNoble{
			Level:   uint16(noble.Level),
			EndTime: noble.EndTime.Unix(),
		}
	}

	//本人，计算，喜欢统计，钻石数量
	cvUserDetail.IsShowAge = uint8(user.IsShowAge)
	cvUserDetail.Birthday = user.Birthday

	//喜欢统计
	var userCount model.UserCount
	err = s.dao.DB.Model(&model.UserCount{}).Where(&model.UserCount{
		UserId: userId,
		Type:   model.CountTypeLikeMe,
	}).First(&userCount).Error
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, err
	}
	cvUserDetail.LikeCount = userCount.Num

	//我喜欢统计
	var userILikeCount model.UserCount
	err = s.dao.DB.Where(&model.UserCount{
		UserId: userId,
		Type:   model.CountTypeLike,
	}).First(&userILikeCount).Error
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, err
	}
	cvUserDetail.ILikeCount = userILikeCount.Num

	//访问统计
	var visitCount int64
	err = s.dao.DB.Model(&model.UserVisit{}).Where(&model.UserVisit{
		VisitUserId: userId,
	}).Count(&visitCount).Error
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, err
	}
	cvUserDetail.VisitCount = int(visitCount)

	//钻石数量
	cvDiamond, err := s.dao.GetDiamond(userId)
	if err != nil {
		return nil, err
	}
	cvUserDetail.DiamondNum = cvDiamond.DiamondNum

	isAgent, _ := s.dao.MgrClient.IsAgent(ctx, &mgr.IsAgentReq{UserId: userId})
	cvUserDetail.IsAgentMgr = isAgent.Ok

	if userTradeUnion == nil {
		cvUserDetail.IsTradeUnion = false
		cvUserDetail.IsTradeUnionMatchNotification = false

	} else {
		cvUserDetail.IsTradeUnion = true
		cvUserDetail.IsTradeUnionMatchNotification = mysql.OpenClose(userTradeUnion.MatchNotification) == mysql.OPEN
	}

	// 永恒之心的值 TODO
	//hv, ok := hvMap[user.ID]
	//if ok {
	//	cvUserDetail.HeartValue = hv.HeartValue
	//	cvUserDetail.HeartValueMax = hv.HeartValueMax
	//	cvUserDetail.MeetDays = hv.MeetDays
	//} else {
	//	cvUserDetail.HeartValueMax = 0
	//}

	cvUserDetail.HeartValueMax = 0
	return cvUserDetail, nil

}

func (s *UserService) getMedalInfoMap(medals map[uint64][]int) (map[uint64][]int, map[uint64][]*model.CvMedal, error) {
	resMedals, err := s.dao.GetAllMedalMap()
	if err != nil {
		return nil, nil, err
	}

	medalIds := make(map[uint64][]int)
	medalMap := make(map[uint64][]*model.CvMedal, 0)

	// 只选择合法的勋章
	for u, i := range medals {
		medalIds[u] = make([]int, 0)
		medalMap[u] = make([]*model.CvMedal, 0)

		for _, j := range i {
			if e, ok := resMedals[j]; ok {
				medalIds[u] = append(medalIds[u], j)
				medalMap[u] = append(medalMap[u], &model.CvMedal{
					Id:        uint32(j),
					PicUrl:    e.PicUrl,
					EffectUrl: e.SvgaUrl,
				})
			}
		}
	}
	return medalIds, medalMap, nil
}

func (s *UserService) GetUserGroupPowerInfo(userId int64) (int64, string, error) {
	gpu, err := s.dao.GetGroupPowerUser(userId)
	if err != nil {
		return 0, "", err
	}
	if gpu == nil || gpu.GroupPowerId == 0 {
		return 0, "", nil
	}
	gp, err := s.dao.GetGroupPower(gpu.GroupPowerId)
	if err != nil {
		return 0, "", err
	}

	powerName := ""
	if gp != nil && len(gp.GroupUid) > 0 {
		gi, err := s.dao.GetGroupInfo(gp.GroupUid)
		if err != nil {
			return 0, "", err
		}
		if gi != nil {
			// 只要前15个字
			s := []rune(gi.Name)
			if len(s) <= 15 {
				powerName = string(s)
			} else {
				powerName = string(s[0:15])
			}
		}
	}
	return gpu.GroupPowerId, powerName, nil
}

func (s *UserService) FindActiveNoble(userId int64) (*model.UserNoble, error) {
	records, err := s.dao.ListNoble(userId)
	if err != nil {
		return nil, err
	}
	if len(records) <= 0 {
		return nil, nil
	}
	return records[0], nil
}

// GetUserTradeUnion 判断是否工会
func (s *UserService) GetUserTradeUnion(userId int64) (*model.UserTradeUnion, error) {
	var userTradeUnion model.UserTradeUnion
	if err := s.dao.DB.Model(&model.UserTradeUnion{
		UserId: userId,
	}).First(&userTradeUnion).Error; err != nil {
		if err == gorm.ErrRecordNotFound {
			return nil, nil
		} else {
			return nil, err
		}
	}
	return &userTradeUnion, nil
}

func (s *UserService) GetSuperManagerMap(userIds []int64) (map[int64]bool, error) {
	if len(userIds) == 0 {
		return map[int64]bool{}, nil
	}
	var superManagers = make([]*model.SuperManager, 0)
	if err := s.dao.DB.Model(&model.SuperManager{}).Where("user_id in (?)", userIds).
		Find(&superManagers).Error; err != nil {
		return nil, err
	}
	//转换成map
	rs := map[int64]bool{}
	for i, _ := range userIds {
		rs[userIds[i]] = false
	}
	for i := range superManagers {
		rs[superManagers[i].UserId] = true
	}
	return rs, nil
}
